@@ -114,7 +114,7 @@ fm_try_callstack <- function(expr) {
       stack <- stack[-(idx + seq_len(6))]
       stack[idx] <- "fm_try_callstack(...)"
     }
-    stack <- paste0(seq_len(length(stack)), ": ", stack, collapse = "\n")
+    stack <- paste0(seq_along(stack), ": ", stack, collapse = "\n")
     assign("error_stack", value = stack, envir = try_envir)
   }
   result <- try(
@@ -134,7 +134,6 @@ fm_try_callstack <- function(expr) {
 }
 
 
-
 fm_require_message <- function(pkg, msg = NULL, override = NULL) {
   if (requireNamespace(pkg, quietly = TRUE)) {
     return(TRUE)
@@ -154,7 +153,7 @@ fm_require_message <- function(pkg, msg = NULL, override = NULL) {
       msg
     )
   )
-  return(FALSE)
+  FALSE
 }
 fm_require_stop <- function(pkg, msg = NULL, override = NULL) {
   if (requireNamespace(pkg, quietly = TRUE)) {
@@ -181,7 +180,6 @@ fm_require_stop <- function(pkg, msg = NULL, override = NULL) {
 }
 
 
-
 #' @title Conversion between sparse matrix types
 #' @rdname fmesher_sparse
 #' @param x Object to be converted
@@ -306,7 +304,6 @@ fm_as_dgTMatrix.fmesher_sparse <- function(x, unique = TRUE, ...) {
 }
 
 
-
 #' Row-wise Kronecker products
 #'
 #' Takes two Matrices and computes the row-wise Kronecker product.  Optionally
@@ -452,7 +449,7 @@ fm_row_kron <- function(M1, M2, repl = NULL, n.repl = NULL, weights = NULL # ,
   )
   #  }
 
-  return(M)
+  M
 }
 
 
@@ -494,7 +491,6 @@ package_methods <- function() {
 }
 
 
-
 fm_capabilities <- function(class = NULL,
                             method = NULL) {
   if (!is.null(class)) {
