@@ -6,7 +6,10 @@ test_that("fm_cprod(..., na.rm = TRUE) sf output can be generated", {
 
   expect_s3_class(ips, "sf")
   expect_equal(nrow(ips), 1)
-  expect_equal(names(ips), c("geometry", "weight", ".block", ".block_origin"))
+  expect_setequal(
+    names(ips),
+    c("geometry", "weight", ".block", ".block_origin")
+  )
   expect_equal(as.numeric(unlist(sf::st_geometry(ips))), c(3, 5))
   expect_equal(as.numeric(ips[["weight"]]), 1)
 })
@@ -18,7 +21,10 @@ test_that("fm_cprod(..., na.rm = FALSE) sf output can be generated", {
 
   expect_s3_class(ips, "sf")
   expect_equal(nrow(ips), 6)
-  expect_equal(names(ips), c("geometry", "weight", ".block", ".block_origin"))
+  expect_setequal(
+    names(ips),
+    c("geometry", "weight", ".block", ".block_origin")
+  )
   expect_equal(
     as.numeric(unlist(sf::st_geometry(ips))),
     c(3, 5, 4, 6, 5, 7, 6, 8, 1, 3, 2, 4)
@@ -39,12 +45,9 @@ test_that("fm_cprod(..., na.rm = FALSE) sf output with different geometry", {
 
   expect_s3_class(ips, "sf")
   expect_equal(nrow(ips), 12)
-  expect_equal(
-    sort(names(ips)),
-    sort(c(
-      "geometry1", "geometry2", "weight",
-      ".block", ".block_origin"
-    ))
+  expect_setequal(
+    names(ips),
+    c("geometry1", "geometry2", "weight", ".block", ".block_origin")
   )
   expect_equal(
     as.numeric(unlist(sf::st_geometry(ips))),
@@ -77,7 +80,10 @@ test_that("fm_cprod(na.rm = TRUE) sp output can be generated", {
 
   expect_s4_class(ips, "Spatial") # or precisely SpatialPointsDataFrame
   expect_equal(nrow(ips), 1)
-  expect_equal(names(ips), c("weight", ".block", ".block_origin"))
+  expect_setequal(
+    names(ips),
+    c("weight", ".block", ".block_origin")
+  )
   expect_equal(as.numeric(sp::coordinates(ips)), c(3, 5))
   expect_equal(as.numeric(ips[["weight"]]), 1)
 })
@@ -94,7 +100,10 @@ test_that("fm_cprod(na.rm = FALSE) sp output can be generated", {
 
   expect_s4_class(ips, "Spatial") # or precisely SpatialPointsDataFrame
   expect_equal(nrow(ips), 6)
-  expect_equal(names(ips), c("weight", ".block", ".block_origin"))
+  expect_setequal(
+    names(ips),
+    c("weight", ".block", ".block_origin")
+  )
   expect_equal(as.numeric(sp::coordinates(ips)), c(3:6, 1:2, 5:8, 3:4))
   expect_equal(as.numeric(ips[["weight"]]), rep(c(1, NA), c(1, 5)))
 })
